@@ -46,12 +46,15 @@ read_cohort_set <- function(path) {
       dplyr::mutate(cohort_definition_id = .data$cohortId, cohort_name = .data$cohortName)
   } else {
     jsonFiles <- sort(list.files(path, pattern = "\\.json$", full.names = TRUE))
+
     cohortsToCreate <- dplyr::tibble(
       cohort_definition_id = seq_along(jsonFiles),
       cohort_name = tools::file_path_sans_ext(basename(jsonFiles)),
       json_path = jsonFiles) %>%
       dplyr::mutate(cohort = purrr::map(.data$json_path, jsonlite::read_json)) %>%
-      dplyr::mutate(json = purrr::map(.data$json_path, readr::read_file))
+      dplyr::mutate(json = purrr::map(.data$json_path, readr::read_file)) %>%
+      dplyr::mutate(cohort_name = stringr::str_replace_all(tolower(.data$cohort_name), "\\s", "_")) %>%
+      dplyr::mutate(cohort_name = stringr::str_remove_all(.data$cohort_name, "[^a-z1-9_]"))
   }
 
   # snakecase name can be used for column names or filenames
@@ -70,7 +73,7 @@ readCohortSet <- read_cohort_set
 #' Generate a cohort set on a cdm object
 #'
 #' @description
-#' A "GeneratedCohortSet" object consists of four components
+#' A "chort_table" object consists of four components
 #' \itemize{
 #'   \item{A remote table reference to an OHDSI cohort table with at least
 #'         the columns: cohort_definition_id, subject_id, cohort_start_date,
@@ -92,9 +95,9 @@ readCohortSet <- read_cohort_set
 #'
 #' `r lifecycle::badge("experimental")`
 #'
-#' One key design principle is that GeneratedCohortSet objects are created once
+#' One key design principle is that cohort_table objects are created once
 #' and can persist across analysis execution but should not be modified after
-#' creation. While it is possible to modify a GeneratedCohortSet object doing
+#' creation. While it is possible to modify a cohort_table object doing
 #' so will invalidate it and it's attributes may no longer be accurate.
 #'
 #' @param cdm A cdm reference created by CDMConnector. write_schema must be
@@ -121,8 +124,8 @@ readCohortSet <- read_cohort_set
 #'
 #' print(cdm$cohort)
 #'
-#' cohortAttrition(cdm$cohort)
-#' cohortSet(cdm$cohort)
+#' attrition(cdm$cohort)
+#' settings(cdm$cohort)
 #' cohortCount(cdm$cohort)
 #' }
 generateCohortSet <- function(cdm,
@@ -130,7 +133,6 @@ generateCohortSet <- function(cdm,
                               name,
                               computeAttrition = TRUE,
                               overwrite = TRUE) {
-
   rlang::check_installed("CirceR")
   rlang::check_installed("SqlRender")
 
@@ -161,7 +163,7 @@ generateCohortSet <- function(cdm,
 
 
   checkmate::assertClass(cdm, "cdm_reference")
-  con <- attr(cdm, "dbcon")
+  con <- cdmCon(cdm)
   checkmate::assertTRUE(DBI::dbIsValid(con))
   checkmate::assert_character(name, len = 1, min.chars = 1, any.missing = FALSE, pattern = "[a-zA-Z0-9_]+")
 
@@ -170,7 +172,7 @@ generateCohortSet <- function(cdm,
   checkmate::assertLogical(computeAttrition, len = 1)
   checkmate::assertLogical(overwrite, len = 1)
 
-  write_schema <- attr(cdm, "write_schema")
+  write_schema <- cdmWriteSchema(cdm)
   checkmate::assert_character(write_schema,
                               min.chars = 1,
                               min.len = 1,
@@ -384,10 +386,13 @@ generateCohortSet <- function(cdm,
 
   # Create attrition attribute ----
   if (computeAttrition) {
-    cohort_attrition_ref <- computeAttritionTable(cdm,
-                                                  cohortStem = name,
-                                                  cohortSet = cohortSet,
-                                                  overwrite = overwrite)
+    cohort_attrition_ref <- computeAttritionTable(
+      cdm = cdm,
+      cohortStem = name,
+      cohortSet = cohortSet,
+      overwrite = overwrite
+    ) |>
+      dplyr::collect()
   } else {
     cohort_attrition_ref <- NULL
   }
@@ -397,41 +402,15 @@ generateCohortSet <- function(cdm,
   #   DBI::dbRemoveTable(con, inSchema(write_schema, paste0(name, "_set"), dbms = dbms(con)))
   # }
 
-  # overwrite not working on snowflake
-  DBI::dbWriteTable(con,
-                    name = inSchema(write_schema, paste0(name, "_set"), dbms(con)),
-                    value = as.data.frame(cohortSet[,c("cohort_definition_id", "cohort_name")]),
-                    overwrite = TRUE)
-
-  cohort_set_ref <- dplyr::tbl(con, inSchema(write_schema, paste0(name, "_set"), dbms(con)))
-
-  # Create cohort_count attribute ----
-  cohort_count_ref <- cohort_ref %>%
-    dplyr::ungroup() %>%
-    dplyr::group_by(.data$cohort_definition_id) %>%
-    dplyr::summarise(number_records = dplyr::n(),
-                     number_subjects = dplyr::n_distinct(.data$subject_id)) %>%
-    {dplyr::left_join(cohort_set_ref, ., by = "cohort_definition_id")} %>%
-    dplyr::mutate(number_records  = dplyr::coalesce(.data$number_records, 0L),
-                  number_subjects = dplyr::coalesce(.data$number_subjects, 0L)) %>%
-    dplyr::select("cohort_definition_id",
-                  "number_records",
-                  "number_subjects") %>%
-    computeQuery(name = paste0(name, "_count"),
-                 schema = attr(cdm, "write_schema"),
-                 temporary = FALSE,
-                 overwrite = TRUE)
-
-  # cohort_ref must be a cdm table before it is passed in to new_generated_cohort_set.
-  cdm[[name]] <- cohort_ref
+  cdm[[name]] <- cohort_ref |>
+    omopgenerics::newCdmTable(src = attr(cdm, "cdm_source"), name = name)
 
+# browser()
   # Create the object. Let the constructor handle getting the counts.----
-  cdm[[name]] <- new_generated_cohort_set(
-    cohort_ref = cdm[[name]],
-    cohort_set_ref = cohort_set_ref,
-    cohort_attrition_ref = cohort_attrition_ref,
-    cohort_count_ref = cohort_count_ref,
-    overwrite = overwrite)
+  cdm[[name]] <- omopgenerics::newCohortTable(
+    table = cdm[[name]],
+    cohortSetRef = cohortSet[,c("cohort_definition_id", "cohort_name")],
+    cohortAttritionRef = cohort_attrition_ref)
 
   cli::cli_progress_done()
 
@@ -451,32 +430,35 @@ generate_cohort_set <- function(cdm,
                     name = name,
                     computeAttrition = compute_attrition,
                     overwrite = overwrite)
-
 }
 
 
-#' Constructor for GeneratedCohortSet objects
+#' Constructor for cohort_table objects
+#'
+#' `r lifecycle::badge("superseded")`
+#'
+#' Please use `omopgenerics::newCohortTable()` instead.
 #'
 #' This constructor function is to be used by analytic package developers to
-#' create `generatedCohortSet` objects.
+#' create `cohort_table` objects.
 #'
 #' @details
-#' A `generatedCohort` is a set of person-time from an OMOP CDM database.
-#' A `generatedCohort` can be represented by a table with three columns:
+#' A `cohort_table` is a set of person-time from an OMOP CDM database.
+#' A `cohort_table` can be represented by a table with three columns:
 #' subject_id, cohort_start_date, cohort_end_date. Subject_id is the same as
-#' person_id in the OMOP CDM. A `generatedCohortSet` is a collection of one
-#' or more `generatedCohorts` and can be represented as a table with four
+#' person_id in the OMOP CDM. A `cohort_table` is a collection of one
+#' or more `cohort_table` and can be represented as a table with four
 #' columns: cohort_definition_id, subject_id, cohort_start_date,
 #' cohort_end_date.
 #'
-#' This constructor function defines the `generatedCohortSet` object in R.
+#' This constructor function defines the `cohort_table` object in R.
 #'
 #' The object is an extension of a `tbl_sql` object defined in dplyr. This is
 #' a lazy database query that points to a cohort table in the database with
 #' at least the columns cohort_definition_id, subject_id, cohort_start_date,
 #' cohort_end_date. The table could optionally have more columns as well.
 #'
-#' In addition the `generatedCohortSet` object has three optional attributes.
+#' In addition the `cohort_table` object has three optional attributes.
 #' These are: cohort_set, cohort_attrition, cohort_count.
 #' Each of these attributes is also a lazy SQL query (`tbl_sql`) that points
 #' to a table in a database and is described below.
@@ -500,11 +482,11 @@ generate_cohort_set <- function(cdm,
 #' ## cohort_count
 #'
 #' cohort_count is a option attribute table that records the number of records
-#' and the number of unique persons in each cohort in a `generatedCohortSet`.
+#' and the number of unique persons in each cohort in a `cohort_table`.
 #' It is derived metadata that can be re-derived as long as cohort_set,
 #' the complete list of cohorts in the set, is available. Column names of
 #' cohort_count are: cohort_definition_id, number_records,
-#' number_subjects. This table is required for generatedCohortSet objects and
+#' number_subjects. This table is required for cohort_table objects and
 #' will be created if not supplied.
 #'
 #' @param cohort_ref,cohortRef A `tbl_sql` object that points to a remote cohort table
@@ -522,13 +504,15 @@ generate_cohort_set <- function(cdm,
 #' cohort_subjects.
 #' @param overwrite Should tables be overwritten if they already exist? TRUE or FALSE (default)
 #'
-#' @return A `generatedCohortSet` object that is a `tbl_sql` reference
+#' @return A `cohort_table` object that is a `tbl_sql` reference
 #' to a cohort table in the write_schema of an OMOP CDM
 #' @export
 #'
+#' @include reexports-omopgenerics.R
+#'
 #' @examples
 #' \dontrun{
-#'  # This function is for developers who are creating generatedCohortSet
+#'  # This function is for developers who are creating cohort_table
 #'  # objects in their packages. The function should accept a cdm_reference
 #'  # object as the first argument and return a cdm_reference object with the
 #'  # cohort table added. The second argument should be `name` which will be
@@ -569,235 +553,23 @@ new_generated_cohort_set <- function(cohort_ref,
                                      cohort_set_ref = NULL,
                                      cohort_attrition_ref = NULL,
                                      cohort_count_ref = NULL,
-                                     overwrite = TRUE) {
-
-  if (!methods::is(cohort_ref, "tbl_sql")) {
-    rlang::abort("cohort_ref must be a remote database table reference (tbl_sql)")
+                                     overwrite) {
+  lifecycle::deprecate_warn(
+    when = "1.3",
+    what = "new_generated_cohort_set()",
+    with = "newCohortTable()"
+  )
+  if (!is.null(cohort_count_ref)) {
+    cli::cli_warn("cohort_count_ref is no longer a required argument for new_generated_cohort_set")
   }
-
-  q <- cohort_ref %>%
-    dbplyr::sql_render() %>%
-    as.character()
-
-  if (stringr::str_detect(q, "\\(") || stringr::str_detect(q, "^SELECT *", negate = TRUE)) {
-    rlang::abort("cohort_ref needs to be a computed table in the database. \nUse `compute_query()` before passing cohort_ref into `new_generated_cohort_set()`")
+  if (!missing(overwrite)) {
+    cli::cli_warn("overwrite is no longer a required argument for new_generated_cohort_set")
   }
-
-  con <- cohort_ref[[1]]$con
-  checkmate::assertTRUE(DBI::dbIsValid(con))
-  cdm <- attr(cohort_ref, "cdm_reference")
-  attr(cohort_ref, "cdm_reference") <- NULL # Very important
-  if (is.null(cdm)) {
-    rlang::abort("cohort_ref must be part of a cdm!")
-  }
-  write_schema <- attr(cdm, "write_schema")
-  if (is.null(write_schema)) {
-    rlang::abort("cohort_ref must be part of a cdm with a write_schema!")
-  }
-  checkmate::assert_character(write_schema, min.len = 1, max.len = 3, min.chars = 1)
-  verify_write_access(con, write_schema = write_schema)
-
-  # cohort table ----
-  {
-    expected_columns <- c("cohort_definition_id", "subject_id", "cohort_start_date", "cohort_end_date")
-    actual_columns <- tolower(colnames(cohort_ref))[1:4]
-    if (!dplyr::setequal(expected_columns, actual_columns)) {
-      rlang::abort(glue::glue("cohort table column names should be {paste(expected_columns, collapse = ', ')} but are {paste(actual_columns, collapse = ', ')}!"))
-    }
-
-    # get the table name from the cohort table. name argument will be ignored.
-    name <- dbplyr::remote_name(cohort_ref)
-    write_prefix <- write_schema["prefix"]
-    if (!is.na(write_prefix)) {
-      if (substr(name, 1, nchar(write_prefix)) != write_prefix) {
-        rlang::abort(glue::glue("cohort_ref ({name}) does not have the same prefix than the write_schema ({write_prefix})"))
-      }
-      name <- substr(name, nchar(write_prefix) + 1, nchar(name))
-    }
-    checkmate::assertCharacter(name, len = 1, min.chars = 1)
-  }
-
-  # we will check that cohort_set contains all the ids in the cohort table
-  cohort_ids <- dplyr::distinct(cohort_ref, .data$cohort_definition_id) %>% dplyr::pull()
-
-  # cohort_set table ----
-  if (is.data.frame(cohort_set_ref)) {
-
-    if ((paste0(name, "_set") %in% list_tables(con, write_schema)) && overwrite) {
-      DBI::dbRemoveTable(con, inSchema(write_schema, paste0(name, "_set"), dbms = dbms(con)))
-    }
-
-    DBI::dbWriteTable(con, name = inSchema(write_schema, paste0(name, "_set"), dbms = dbms(con)), cohort_set_ref, overwrite = overwrite)
-    cohort_set_ref <- dplyr::tbl(con, inSchema(write_schema, paste0(name, "_set"), dbms = dbms(con))) %>%
-      dplyr::rename_all(tolower)
-
-  } else if (is.null(cohort_set_ref)) {
-
-    # create the cohort_set table
-    cohort_set_ref <- cohort_ref %>%
-      dplyr::distinct(.data$cohort_definition_id) %>%
-      dplyr::mutate(cohort_name = paste0("cohort_", .data$cohort_definition_id)) %>%
-      computeQuery(
-        name = paste0(name, "_set"),
-        temporary = FALSE,
-        schema = write_schema,
-        overwrite = overwrite)
-  }
-
-  {
-    if (!methods::is(cohort_set_ref, "tbl_sql")) {
-      rlang::abort("cohort_set_ref must be a dataframe or a remote table reference")
-    }
-
-    nm <- dbplyr::remote_name(cohort_set_ref)
-    if (!is.na(write_prefix)) {
-      if (substr(nm, 1, nchar(write_prefix)) != write_prefix) {
-        rlang::abort(glue::glue("cohort_set_ref ({nm}) does not have the same prefix than the write_schema ({write_prefix})"))
-      }
-      nm <- substr(nm, nchar(write_prefix) + 1, nchar(nm))
-    }
-    name_set <- paste0(name, "_set")
-    if (nm != name_set) {
-      rlang::abort(glue::glue("cohort_set_ref database table name is {nm} but should be {name_set}!"))
-    }
-
-    expected_columns <- c("cohort_definition_id", "cohort_name")
-    actual_columns <- tolower(colnames(cohort_set_ref))[1:2]
-    if (!dplyr::setequal(expected_columns, actual_columns)) {
-      rlang::abort(glue::glue("cohort_set column names should be `cohort_definition_id` `cohort_name` but are {paste(actual_columns, collapse = ', ')}!"))
-    }
-
-    # primary key check
-    if(cohort_set_ref %>% utils::head(1) %>% dplyr::tally() %>% dplyr::pull() != 0){
-      one <- cohort_set_ref %>%
-      dplyr::count(.data$cohort_definition_id, name = "one") %>%
-      dplyr::count(.data$one) %>%
-      dplyr::pull("one")
-
-    if (!(length(one) == 1 && one == 1)) {
-      rlang::abort("cohort_definition_id is not a primary key on cohort_set_ref!")
-    }
-
-    cohort_set_ids <- dplyr::pull(cohort_set_ref, "cohort_definition_id")
-    if (!all(cohort_ids %in% cohort_set_ids)) {
-      diff <- dplyr::setdiff(cohort_ids, cohort_set_ids) %>% paste(collapse = ", ")
-      rlang::abort(glue::glue("cohort IDs {diff} are in the cohort table but not in the cohort_set table!"))
-    }
-    }
-  }
-
-  # cohort_count_table ----
-  if (is.data.frame(cohort_count_ref)) {
-
-    DBI::dbWriteTable(con, name = inSchema(write_schema, paste0(name, "_count"), dbms = dbms(con)), cohort_count_ref, overwrite = overwrite)
-    cohort_count_ref <- dplyr::tbl(con, inSchema(write_schema, paste0(name, "_count"), dbms = dbms(con))) %>%
-      dplyr::rename_all(tolower)
-
-  } else if (is.null(cohort_count_ref)) {
-    # create the cohort_count table
-    cohort_count_ref <- cohort_ref %>%
-      dplyr::group_by(.data$cohort_definition_id) %>%
-      dplyr::summarise(
-        number_records = dplyr::n(),
-        number_subjects = dplyr::n_distinct(.data$subject_id)) %>%
-      dplyr::right_join(
-        dplyr::select(cohort_set_ref, "cohort_definition_id"),
-        by = "cohort_definition_id") %>%
-      dplyr::mutate(
-        number_records = dplyr::coalesce(.data$number_records, 0),
-        number_subjects = dplyr::coalesce(.data$number_subjects, 0)) %>%
-      computeQuery(
-        name = paste0(name, "_count"),
-        temporary = FALSE,
-        schema = write_schema,
-        overwrite = overwrite)
-  }
-
-  {
-    checkmate::assert_class(cohort_count_ref, "tbl_sql")
-
-    nm <- dbplyr::remote_name(cohort_count_ref)
-    if (!is.na(write_prefix)) {
-      if (substr(nm, 1, nchar(write_prefix)) != write_prefix) {
-        rlang::abort(glue::glue("cohort_count_ref ({nm}) does not have the same prefix than the write_schema ({write_prefix})"))
-      }
-      nm <- substr(nm, nchar(write_prefix) + 1, nchar(nm))
-    }
-    name_count <- paste0(name, "_count")
-    if (nm != name_count) {
-      rlang::abort(glue::glue("cohort_count_ref database table name is {nm} but should be {name_count}!"))
-    }
-
-    expected_columns <- c("cohort_definition_id", "number_records", "number_subjects")
-    actual_columns <- tolower(colnames(cohort_count_ref))[1:3]
-    if (!dplyr::setequal(actual_columns, expected_columns)) {
-      rlang::abort(glue::glue("cohort_set column names should be {paste(expected_columns, collapse = ', ')} but are {paste(actual_columns, collapse = ', ')}!"))
-    }
-  }
-
-  # cohort_attrition table ----
-  if (is.data.frame(cohort_attrition_ref)) {
-
-    DBI::dbWriteTable(con, name = inSchema(write_schema, paste0(name, "_attrition"), dbms = dbms(con)), cohort_attrition_ref, overwrite = overwrite)
-    cohort_attrition_ref <- dplyr::tbl(con, inSchema(write_schema, paste0(name, "_attrition"), dbms = dbms(con))) %>%
-      dplyr::rename_all(tolower)
-
-  } else if (is.null(cohort_attrition_ref)) {
-
-    # create a brand new attrition table
-    cohort_attrition_ref <- cohort_count_ref %>%
-      dplyr::mutate(
-        reason = "Qualifying initial records",
-        number_records = dplyr::coalesce(.data$number_records, 0),
-        number_subjects = dplyr::coalesce(.data$number_subjects, 0),
-        reason_id = 1, excluded_records = 0, excluded_subjects = 0) %>%
-        dplyr::select(
-          "cohort_definition_id", "number_records", "number_subjects",
-          "reason_id", "reason", "excluded_records", "excluded_subjects") %>%
-        computeQuery(
-          name = paste0(name, "_attrition"),
-          temporary = FALSE,
-          schema = write_schema,
-          overwrite = overwrite
-        )
-  }
-
-  {
-    checkmate::assert_class(cohort_attrition_ref, "tbl_sql")
-
-    nm <- nm <- dbplyr::remote_name(cohort_attrition_ref)
-
-    if (!is.na(write_prefix)) {
-      if (substr(nm, 1, nchar(write_prefix)) != write_prefix) {
-        rlang::abort(glue::glue("cohort_attrition_ref ({nm}) does not have the same prefix than the write_schema ({write_prefix})"))
-      }
-      nm <- substr(nm, nchar(write_prefix) + 1, nchar(nm))
-    }
-    name_attrition <- paste0(name, "_attrition")
-    if (nm != name_attrition) {
-      rlang::abort(glue::glue("cohort_attrition_ref database table name is {nm} but should be {name_attrition}!"))
-    }
-
-    expected_columns <- c("cohort_definition_id", "number_records", "number_subjects",
-                          "reason_id", "reason", "excluded_records", "excluded_subjects")
-    actual_columns <- tolower(colnames(cohort_attrition_ref))[1:7]
-
-    if (!dplyr::setequal(actual_columns, expected_columns)) {
-      rlang::abort(glue::glue("cohort_attrition column names should be {paste(expected_columns, collapse = ', ')} but are {paste(actual_columns, collapse = ', ')}!"))
-    }
-  }
-
-  # create the GeneratedCohortSet object ----
-  if (!("GeneratedCohortSet" %in% class(cohort_ref))) {
-    class(cohort_ref) <- c("GeneratedCohortSet", class(cohort_ref))
-  }
-
-  attr(cohort_ref, "cohort_set") <- cohort_set_ref
-  attr(cohort_ref, "cohort_attrition") <- cohort_attrition_ref
-  attr(cohort_ref, "cohort_count") <- cohort_count_ref
-  attr(cohort_ref, "tbl_name") <- name
-
-  return(cohort_ref)
+  omopgenerics::newCohortTable(
+    table = cohort_ref,
+    cohortSetRef = cohort_set_ref,
+    cohortAttritionRef = cohort_attrition_ref
+  )
 }
 
 
@@ -807,76 +579,76 @@ newGeneratedCohortSet <- function(cohortRef,
                                   cohortSetRef = NULL,
                                   cohortAttritionRef = NULL,
                                   cohortCountRef = NULL,
-                                  overwrite = FALSE) {
-  new_generated_cohort_set(cohort_ref = cohortRef,
-                           cohort_set_ref = cohortSetRef,
-                           cohort_attrition_ref = cohortAttritionRef,
-                           cohort_count_ref = cohortCountRef,
-                           overwrite = overwrite
+                                  overwrite) {
+  if (!missing(overwrite)) {
+    cli::cli_warn("overwrite is no longer a required argument for new_generated_cohort_set")
+  }
+  new_generated_cohort_set(
+    cohort_ref = cohortRef,
+    cohort_set_ref = cohortSetRef,
+    cohort_attrition_ref = cohortAttritionRef,
+    cohort_count_ref = cohortCountRef
   )
 }
 
 
-#' Get attrition table from a GeneratedCohortSet object
+#' Get attrition table from a cohort_table object
 #'
-#' @param x A generatedCohortSet object
+#' @param x A cohort_table object
 #'
 #' @export
-cohortAttrition <- function(x) { UseMethod("cohortAttrition") }
+cohortAttrition <- function(x) {
+  lifecycle::deprecate_warn("1.3", "cohortAttrition()", "attrition()")
+  omopgenerics::attrition(x)
+}
 
 
 #' @rdname cohortAttrition
 #' @export
-cohort_attrition <- cohortAttrition
-
-
-#' @export
-cohortAttrition.GeneratedCohortSet <- function(x) {
-  if (is.null(attr(x, "cohort_attrition"))) {
-    return(invisible(NULL))
-  } else {
-    a <- dplyr::collect(attr(x, "cohort_attrition"))
-    class(a) <- c("omop_attrition", "attrition", class(a))
-    return(a)
-  }
+cohort_attrition <- function(x) {
+  lifecycle::deprecate_warn("1.3", "cohort_attrition()", "attrition()")
+  omopgenerics::attrition(x)
 }
 
-#' Get cohort settings from a GeneratedCohortSet object
+#' Get cohort settings from a cohort_table object
 #'
-#' @param x A generatedCohortSet object
+#' @param x A cohort_table object
 #'
 #' @export
-cohortSet <- function(x) { UseMethod("cohortSet") }
+cohortSet <- function(x) {
+  lifecycle::deprecate_warn("1.3", "cohortSet()", "settings()")
+  omopgenerics::settings(x)
+}
 
 
 #' @rdname cohortSet
 #' @export
-cohort_set <- cohortSet
-
-
-#' @export
-cohortSet.GeneratedCohortSet <- function(x) {
-  dplyr::collect(attr(x, "cohort_set"))
+cohort_set <- function(x) {
+  lifecycle::deprecate_warn("1.3", "cohort_set()", "settings()")
+  omopgenerics::settings(x)
 }
 
-#' Get cohort counts from a GeneratedCohortSet object
+#' Get cohort counts from a generated_cohort_set object.
 #'
-#' @param x A generatedCohortSet object
+#' @param cohort A generated_cohort_set object.
 #'
+#' @return A table with the counts.
+#' @rdname cohort_count
 #' @export
-cohortCount <- function(x) { UseMethod("cohortCount") }
-
-
-#' @rdname cohortCount
-#' @export
-cohort_count <- cohortCount
-
-
-#' @export
-cohortCount.GeneratedCohortSet <- function(x) {
-  dplyr::collect(attr(x, "cohort_count"))
-}
-
+#'
+#' @examples
+#' \dontrun{
+#' library(CDMConnector)
+#' library(dplyr)
+#'
+#' con <- DBI::dbConnect(duckdb::duckdb(), eunomia_dir())
+#' cdm <- cdm_from_con(con = con, cdm_schema = "main", write_schema = "main")
+#' cdm <- generateConceptCohortSet(
+#'   cdm = cdm, conceptSet = list(pharyngitis = 4112343), name = "new_cohort"
+#' )
+#' cohort_count(cdm$new_cohort)
+#' }
+cohort_count <- omopgenerics::cohortCount
 
 # Compute the attrition for a set of cohorts (internal function)
 #
@@ -912,16 +684,16 @@ computeAttritionTable <- function(cdm,
   checkmate::assertNumeric(cohortId, any.missing = FALSE, min.len = 1)
   checkmate::assertTRUE(all(cohortId %in% cohortSet$cohort_definition_id))
 
-  con <- attr(cdm, "dbcon")
+  con <- cdmCon(cdm)
   checkmate::assertTRUE(DBI::dbIsValid(con))
 
   inclusionResultTableName <- paste0(cohortStem, "_inclusion_result")
 
-  # if (dbms(attr(cdm, "dbcon")) %in% c("oracle", "snowflake")) {
+  # if (dbms(cdmCon(cdm)) %in% c("oracle", "snowflake")) {
   #   inclusionResultTableName <- toupper(inclusionResultTableName)
   # }
 
-  schema <- attr(cdm, "write_schema")
+  schema <- cdmWriteSchema(cdm)
   checkmate::assertCharacter(schema, min.len = 1, max.len = 3, min.chars = 1)
 
   if (paste0(cohortStem, "_attrition") %in% listTables(con, schema = schema)) {
@@ -1076,7 +848,8 @@ caprConceptToDataframe <- function(x) {
   )
 }
 
-#' Add attrition reason to a GeneratedCohortSet object
+
+#' Add attrition reason to a cohort_table object
 #'
 #' Update the cohort attrition table with new counts and a reason for attrition.
 #'
@@ -1102,7 +875,7 @@ caprConceptToDataframe <- function(x) {
 #'   cdm = cdm, conceptSet = list(pharyngitis = 4112343), name = "new_cohort"
 #' )
 #'
-#' cohortSet(cdm$new_cohort)
+#' settings(cdm$new_cohort)
 #' cohortCount(cdm$new_cohort)
 #' cohortAttrition(cdm$new_cohort)
 #'
@@ -1113,108 +886,19 @@ caprConceptToDataframe <- function(x) {
 #'   cohort = cdm$new_cohort, reason = "Only events after 2010"
 #' )
 #'
-#' cohortSet(cdm$new_cohort)
+#' settings(cdm$new_cohort)
 #' cohortCount(cdm$new_cohort)
 #' cohortAttrition(cdm$new_cohort)
 #' }
 recordCohortAttrition <- function(cohort,
                                   reason,
                                   cohortId = NULL) {
-  checkmate::assertClass(cohort, "GeneratedCohortSet")
-  name <- attr(cohort, "tbl_name")
-  checkmate::assertCharacter(name, len = 1, min.chars = 1)
-  checkmate::assertCharacter(reason, len = 1, min.chars = 1, any.missing = FALSE)
-  checkmate::assertIntegerish(cohortId, any.missing = FALSE, null.ok = TRUE)
-
-  cdm <- attr(cohort, "cdm_reference")
-  checkmate::assertClass(cdm, "cdm_reference")
-
-  if (is.null(cohortId)) {
-    cohortId <- attr(cohort, "cohort_set") %>%
-      dplyr::pull("cohort_definition_id")
-  }
-
-  tm <- as.integer(Sys.time())
-
-  tempCohortCount <- attr(cohort, "cohort_count") %>%
-    dplyr::filter(!(.data$cohort_definition_id %in% .env$cohortId)) %>%
-    computeQuery(
-      name = paste0("temp", tm, "a_"),
-      # temporary = getOption("intermediate_as_temp", TRUE),
-      temporary = TRUE,
-      overwrite = TRUE,
-      schema = attr(cdm, "write_schema"))
-
-  # update cohort_count ----
-  attr(cohort, "cohort_count") <- cohort %>%
-    dplyr::filter(.data$cohort_definition_id %in% .env$cohortId) %>%
-    dplyr::group_by(.data$cohort_definition_id) %>%
-    dplyr::summarise(
-      number_records = dplyr::n(),
-      number_subjects = dplyr::n_distinct(.data$subject_id)
-    ) %>%
-    dplyr::union_all(tempCohortCount) %>%
-    dplyr::right_join(
-      attr(cohort, "cohort_set") %>% dplyr::select("cohort_definition_id"),
-      by = "cohort_definition_id"
-    ) %>%
-    dplyr::mutate(
-      number_records = dplyr::coalesce(.data$number_records, 0),
-      number_subjects = dplyr::coalesce(.data$number_subjects, 0)) %>%
-    computeQuery(
-      name = paste0(name, "_count"),
-      temporary = FALSE,
-      schema = attr(cdm, "write_schema"),
-      overwrite = TRUE
-    )
-
-  # update cohort_attrition ----
-  newAttritionRow <- attr(cohort, "cohort_attrition") %>%
-    dplyr::filter(.data$cohort_definition_id %in% .env$cohortId) %>%
-    dplyr::select("cohort_definition_id",
-                  "reason_id",
-                  "previous_records" = "number_records",
-                  "previous_subjects" = "number_subjects") %>%
-    dplyr::group_by(.data$cohort_definition_id) %>%
-    dplyr::filter(.data$reason_id == max(.data$reason_id, na.rm = TRUE)) %>%
-    dplyr::ungroup() %>%
-    dplyr::inner_join(attr(cohort, "cohort_count"), by = c("cohort_definition_id")) %>%
-    dplyr::mutate(
-      reason_id = .data$reason_id + 1,
-      reason = .env$reason,
-      excluded_records = .data$previous_records - .data$number_records,
-      excluded_subjects = .data$previous_subjects - .data$number_subjects) %>%
-    dplyr::select(
-      "cohort_definition_id", "number_records", "number_subjects",
-      "reason_id", "reason", "excluded_records", "excluded_subjects") %>%
-    computeQuery(
-      # temporary = getOption("intermediate_as_temp", TRUE),
-      name = paste0("temp", tm, "b_"),
-      temporary = TRUE,
-      overwrite = TRUE,
-      schema = attr(cdm, "write_schema"))
-
-  tempCohortAttrition <- attr(cohort, "cohort_attrition") %>%
-    computeQuery(
-      # temporary = getOption("intermediate_as_temp", TRUE),
-      name = paste0("temp", tm, "c_"),
-      temporary = TRUE,
-      overwrite = TRUE,
-      schema = attr(cdm, "write_schema"))
-
-  # note that overwrite will drop the table that is needed for the query.
-  # TODO support overwrite existing table using rename in computeQuery. Cross platform table rename is needed for this though.
-  attr(cohort, "cohort_attrition")  <- tempCohortAttrition %>%
-    dplyr::union_all(newAttritionRow) %>%
-    computeQuery(
-      name = paste0(name, "_attrition"),
-      temporary = FALSE,
-      schema = attr(cdm, "write_schema"),
-      overwrite = TRUE)
-
-  return(cohort)
+  omopgenerics::recordCohortAttrition(cohort = cohort,
+                                      reason = reason,
+                                      cohortId = cohortId)
 }
 
 #' @export
 #' @rdname recordCohortAttrition
 record_cohort_attrition <- recordCohortAttrition
+
