@@ -4,8 +4,10 @@
 
 test_that("memory leak does not happen", {
   skip_if_not_installed("duckdb")
-  con <- DBI::dbConnect(duckdb::duckdb(), eunomia_dir())
-  cdm <- cdm_from_con(con, "main", "main")
+  con <- DBI::dbConnect(duckdb::duckdb(eunomia_dir()))
+  cdm <- cdm_from_con(
+    con = con, cdm_name = "eunomia", cdm_schema = "main", write_schema = "main"
+  )
 
   conceptSet <- list(asthma = 317009)
 
